import os


class Container():

    def __init__(self, name, task, context={}, remove=False):
        self.name = name
        self.remove = remove
        self.context = context
        self.task = task
        data = self.task.container_data[self.name]
        self.directory = self.task.directory
        self.image = data.get('image')
        self.build = data.get('build')
        if self.build and not os.path.isabs(self.build):
            self.build = os.path.join(self.directory, self.build)
        self.dockerfile = data.get('dockerfile')
        if self.dockerfile and not os.path.isabs(self.dockerfile):
            self.dockerfile = os.path.join(self.directory, self.dockerfile)
        self.links = [l for l in data.get('links', [])]
        self.volumes = [v for v in data.get('volumes', [])]
        self.volumes_from = [vf for vf in data.get('volumes-from', [])]
        self.ports = [p for p in data.get('ports', [])]
        self.environment = [e for e in data.get('environment', [])]
        self.entrypoint = data.get('entrypoint')
        self.command = data.get('command')

    def get_run_command(self):

        options = ['--name %s-%s' % (self.task.name, self.name)]
        for l in self.links:
            if not len(l.split(':')) > 1:
                l = '%s:%s' % (l, l)
            options.append('--link %s-%s' % (self.task.name, l))
        for _v in self.volumes:
            v = str(_v).format(**self.context)
            if not os.path.isabs(v.split(':')[0]):
                first, last = v.split(':', 1)
                first = os.path.join(self.directory, first)
                v = first + ':' + last
            options.append('-v %s' % v)
        for vf in self.volumes_from:
            options.append('--volumes-from %s' % vf)
        for p in self.ports:
            options.append('-p %s' % p)
        for e in self.environment:
            options.append('-e %s' % e.format(**self.context))
        if self.entrypoint:
            options.append('--entrypoint %s' % self.entrypoint.format(**self.context))
        if self.remove:
            options.append('--rm')
        else:
            options.append('-d')
        options.append(self.image.format(**self.context))
        if self.command:
            options.append(self.command.format(**self.context))
        return 'docker run %s' % ' '.join(options)

    def get_cleanup_command(self):

        return 'docker rm -fv %s-%s' % (self.task.name, self.name)

    def get_setup_command(self):
        if self.build:
            self.image = '%s/%s' % (self.task.name, self.name)
            command = self.image
            if self.dockerfile:
                command = self.image + ' -f %s' % self.dockerfile
            return 'docker build -t %s %s' % (command, self.build)
        else:
            return None
