from wharfrat.container import Container
from wharfrat.utils import read_yaml
import os


class Task():

    def __init__(self, name, data, context={}):
        self._container_data = None
        self.compose_file = data.get('compose_file', 'docker-compose.yml')
        self.directory = os.path.abspath(os.path.dirname(self.compose_file))
        if not os.path.isabs(self.compose_file):
            self.compose_file = os.path.join(self.directory, self.compose_file)

        self.name = name
        self.container_names = data.get('images', [])
        ctx_string = data.get('context', [])
        self.context = {ln.split('=')[0]: ln.split('=')[1] for ln in ctx_string}
        self.context.update(context)
        self._containers = None
        self.primary_name = data.get('primary')
        self._primary = None

    def run(self):
        commands = []
        for x in self.containers:
            if x.get_setup_command():
                commands.append(x.get_setup_command())
        if self.primary:
            if self.primary.get_setup_command():
                commands.append(self.primary.get_setup_command())
        for x in self.containers:
            commands.append(x.get_run_command())
        if self.primary:
            commands.append(self.primary.get_run_command())
        for x in self.containers:
            commands.append(x.get_cleanup_command())
        return commands

    @property
    def container_data(self):
        if not self._container_data:
            self._container_data = read_yaml(self.compose_file)
        return self._container_data

    @property
    def containers(self):
        if not self._containers:
            self._containers = [
                Container(_name, self, context=self.context)
                for _name in self.container_names]
        return self._containers

    @property
    def primary(self):
        if not self._primary and self.primary_name:
            self._primary = Container(
                self.primary_name,
                self, context=self.context, remove=True)
        return self._primary
